#ifndef CGIDATA_H_
#define CGIDATA_H_ 

/*
 * Definition of class CgiData
 * Updated $Date: 1997/03/03 11:07:15 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: CgiData.H,v $
 * Revision 1.7  1997/03/03 11:07:15  stig
 * Major update
 *
 *
 * $Id: CgiData.H,v 1.7 1997/03/03 11:07:15 stig Exp stig $
 */

#include <limits.h>
#include <cgi/CgiEnv.H>

class CgiData {
public:
  CgiData( );
  ~CgiData( void );
  
  inline CgiEnv* getCgiEnv( void ) const;

  char *getvar(char *name, int max, int &found, int newlines=1);
  char** getmultivar( char *name, int& numFound );
  int getint(char *name, int &found);
  double getdouble( char *name, int& found );

protected:
  inline FormEntry* findEntry( const char *name ) const;
  FormEntry* findNextEntry(const char *findTarget, FormEntry *findPos) const;

  int makeString( FormEntry *e, char *result, int max, int newlines) const;
  int unEscapeChars( char **sp, const char *cp, int len ) const;
  int firstNonspaceChar(const char *s ) const;

  inline int getFormInput( void );
  inline int postFormInput( void );
  int parseFormInput( const char *data, int length );
  
  inline FormEntry* getFirstEntry( void ) const;

private:
  FormEntry		*itsFirstEntry;
  CgiEnv        	*itsEnv;
};

//Inline Functions:

inline int
CgiData::getFormInput( void ){
  /*SDS
   * Parses GET data
   */
  
  return parseFormInput( itsEnv->getQueryString(), 
			 itsEnv->getContentLength() );
};

inline int
CgiData::postFormInput( void ){
  /*SDS
   * Parses POST data
   */
  
  return parseFormInput( itsEnv->getPostData(), 
			 itsEnv->getContentLength() );
};

inline FormEntry*
CgiData::getFirstEntry( void ) const {
  /*SDS
   * Retrieves the first entry
   */
  return itsFirstEntry;
};

inline FormEntry*
CgiData::findEntry( const char *name ) const {

  /*SDS
   * Function to find an entry. Calls the general
   * findNextEntry(..)
   */

  if (!name) return NULL;
  return findNextEntry(name, itsFirstEntry);
};


inline CgiEnv*
CgiData::getCgiEnv( void ) const{
  /*SDS
   * Returns a pointer to the environment
   */
  
  return itsEnv;
}

#endif


