#ifndef __CGIPP_H_
#define __CGIPP_H_ 

/*
 * Cgipp.cpp
 *
 * The Cgipp library, by Stig Erik Sand, version 0.3 is a major
 * upgrade from versions 0.2 and 0.1.x. The 0.1.x versions were
 * more or less a patched/more portable/less error frequent/less
 * memory demanding version of the Cgicc library, by Stephen Booth.
 * The upgrade from v0.1.x to v0.2 reduced the number of classes
 * and made it a whole lot safer.
 * The upgrade from 0.2 to 0.3 largely consists of better
 * standardization and reduction in functions. Effort has not been made
 * to make earlier versions compatible with Cgipp v0.3
 *  
 * Cgicc is a C++ version of the C library cgic by Thomas Boutell,
 * available from sunsite.unc.edu/boutell/cgic/cgic.html
 *  
 * Cgipp/Cgicc is intended to simplify and speed the development of C++
 * cgi(common gateway interface) applications by providing a simple,
 * user friendly set of classes that present full cgi functionality.
 *
 * Version 0.2.x has been tested with gcc 2.7.x and libg++ 2.7.x
 * on Linux 1.2.13, IRIX 5.3, DEC OSF/1, Solaris 2.4, Solaris 2.5,
 * SunOS 4.1.3. It has also been compiled succesfully with Borland C++
 * and SGI C++ for IRIX 6.3. It should compile flawlessly on these
 * platforms. Minor changes needed only in the Makefile
 *
 * $Id: Cgipp.H,v 1.4 1997/03/03 11:09:12 stig Exp stig $
 */

#include <cgi/CgiEnv.H>
#include <cgi/htmlTool.H>

class Cgipp {
public:  
  Cgipp();
  ~Cgipp();

  inline CgiEnv* getCgiEnv( void ) const;

  char *getvar(char *name, int max, int &found, int newlines=1);
  char** getmultivar( char *name, int& numFound );
  int getint(char *name, int &found);
  double getdouble( char *name, int& found );
  
protected:
  inline FormEntry* findEntry( const char *name ) const;
  FormEntry* findNextEntry(const char *findTarget, FormEntry *findPos) const;

  int makeString( FormEntry *e, char *result, int max, int newlines) const;
  int unEscapeChars( char **sp, const char *cp, int len ) const;
  int firstNonspaceChar(const char *s ) const;

  inline int getFormInput( void );
  inline int postFormInput( void );
  int parseFormInput( const char *data, int length );
  
  inline FormEntry* getFirstEntry( void ) const;

private:
  FormEntry		*itsFirstEntry;
  CgiEnv        	*itsEnv;

};

//Inline Functions:

inline int
Cgipp::getFormInput( void ){
  /*SDS
   * Parses GET data
   */
  
  return parseFormInput( itsEnv->getQueryString(), 
			 itsEnv->getContentLength() );
};

inline int
Cgipp::postFormInput( void ){
  /*SDS
   * Parses POST data
   */
  
  return parseFormInput( itsEnv->getPostData(), 
			 itsEnv->getContentLength() );
};

inline FormEntry*
Cgipp::getFirstEntry( void ) const {
  /*SDS
   * Retrieves the first entry
   */
  return itsFirstEntry;
};

inline FormEntry*
Cgipp::findEntry( const char *name ) const {

  /*SDS
   * Function to find an entry. Calls the general
   * findNextEntry(..)
   */

  if (!name) return NULL;
  return findNextEntry(name, itsFirstEntry);
};


inline CgiEnv*
Cgipp::getCgiEnv( void ) const{
  /*SDS
   * Returns a pointer to the environment
   */
  
  return itsEnv;
}


#endif /* Cgipp */



