#ifndef CGIDEBUG_H_
#define CGIDEBUG_H_ 

#ifdef unix
#include <cgi/sysdefs.H>
#else
#include <cgi/pcdefs.H>
#endif
#include <fstream.h>

/* Definition of Debugging/Error Handling
 * Updated $Date: 1997/03/03 11:08:41 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: Debug.H,v $
 * Revision 1.3  1997/03/03 11:08:41  stig
 * Major update
 *
 *
 * $Id: Debug.H,v 1.3 1997/03/03 11:08:41 stig Exp stig $
 */

/*
 * ---------------------------------------------------------------
 * Debugging, the difficult part in cgi-programs
 * ---------------------------------------------------------------
 */

#if defined(EBUG) || defined(DEBUG)
#ifndef DEBUG
#define DEBUG 1
#endif
#ifndef CGIPPDEBUG
#define CGIPPDEBUG 1
#endif
#endif /* def ebug || def debug */

#ifdef MAINTAINER
#define MAINT 1
#else
#define MAINT 0
#endif

#ifdef NODEBUG
#ifdef MAINT
#undef MAINT
#define MAINT 0
#endif
#ifdef DEBUG
#undef DEBUG
#endif
#ifdef CGIPPDEBUG
#undef CGIPPDEBUG
#endif
#endif /* NODEBUG */

// A small neat debugging class

class DebugTool : public ofstream {
public:
  DebugTool (void);
  ~DebugTool(void);
};

// We put it somewhere else..

extern DebugTool cgidebug;

// Let us define what we need
#ifndef DEBUGOUTPUT
#define DEBUGOUTPUT cgidebug
#endif

/*
 * If you define CGIPPDEBUG (e.g. define EBUG on command-line) you'll
 * get full debugging capabilities.  When you do not want debugging anymore,
 * the debugging code is removed automagically
 */

#ifdef CGIPPDEBUG

// Debugging is tough when cgi-programming, these macros may help
#define DEBUGPUT(arg) DEBUGOUTPUT << "File: " << __FILE__ << ", Line: " << __LINE__ << " # " <<  arg << endl
#define _DEBUGPUT(myostream, arg) myostream << "File: " << __FILE__ << ", Line: " << __LINE__ << " # " <<  arg << endl

// A few _very_ useful macros for tracing crashes
// Be very careful around if's, for and while's, etc...

#define TRACE(arg) DEBUGOUTPUT << #arg << endl; arg
#define _TRACE(myostream, arg) myostream << #arg << endl; arg

// A simple line is surprisingly useful when reading debug-info.

#define LINEPUT() DEBUGOUTPUT << "------------------------------------" << endl

#else /* that is, if we're not debugging */

#define DEBUGPUT(arg)
#define _DEBUGPUT(arg)
#define LINEPUT() 

// We execute the trace but print nought.
#define TRACE(arg) arg
#define _TRACE(myostream, arg) arg

#endif

/*
 * ---------------------------------------------------------------
 * Error-handling/reporting functions. Not very developed.
 * ---------------------------------------------------------------
 */

// Crash and Burn with the neon lights on :-)

int cgiError(const char *msg, int line, const char *err_file); 
#define _CgiError(arg1) cgiError(arg1, __LINE__, __FILE__) 

#endif /* DEBUG_H */

