#ifndef CGIDEFS_H_
#define CGIDEFS_H_ 

/*
 * Definitions for full CGI-Library
 * Updated $Date: 1997/03/03 11:03:01 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: Defs.H,v $
 * Revision 1.6  1997/03/03 11:03:01  stig
 * major update
 *
 *
 * $Id: Defs.H,v 1.6 1997/03/03 11:03:01 stig Exp stig $
 */

#ifdef unix
#include <cgi/sysdefs.H>
#else
#include <cgi/pcdefs.H>
#endif

#ifndef NOTEMPLATES /* Not used yet */
#define TEMPLATES 1
#endif

#ifndef logFile
#define logFile "/dev/null"
#endif

#ifndef admin
#define admin "webmaster"
#endif

#ifndef CGIPARSESIZE
#define CGIPARSESIZE 1024
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include <cgi/Debug.H>

#ifndef CGITRUE
#define CGITRUE 1
#define CGIFALSE 0
#define CGIOK 1
#endif

// Basic linked list class
class FormEntry {
public:
  FormEntry();
  FormEntry(const char *attrib, const char *val);
  FormEntry(const FormEntry &entry);
  FormEntry &operator=(const FormEntry &entry);
  ~FormEntry();

  // Observers
  inline int Last() const; // Returns if next is NULL
  inline const char *Value() const;
  inline const char *Attr() const;
  inline FormEntry *Next() const;
  void dumpall(void) const;

  // Updating methods
  int setvalues(const char *attrib, const char *val);
  inline void setNext(FormEntry *nextentry);
private:
  char	    *attr;
  char	    *value;
  FormEntry *next;
};

// Useful macros that I didn't want in a class

#ifdef HAVE_STRCASECMP
#define noCaseCompare(arg1,arg2) (!strcasecmp((arg1),(arg2)))
#else
#define noCaseCompare(arg1,arg2) (!stricmp((arg1),(arg2)))
#endif
 
inline int
FormEntry::Last() const{
  
  /*SDS
   * True if next is NULL
   */
  
  return (!next);
};

inline const char *
FormEntry::Value() const {
  /*SDS
   * Returns the value of the variable
   */
  return value;
};

inline const char *
FormEntry::Attr() const {
  /*SDS
   * Returns the name of the variable
   */
  
  return attr;
};

inline FormEntry *
FormEntry::Next() const{
  /*SDS
   * Returns the next object
   */
  
  return next;
};

inline void
FormEntry::setNext(FormEntry *nextentry){
  /*SDS
   * Sets the next pointer to point at nextentry
   */
  if (nextentry)
    next=nextentry;
  else
    next=NULL;
};

#endif


