#ifndef TOOLBOX_H_
#define TOOLBOX_H_

/*
 * Toolbox - Headerfile
 *
 * $Log: Tools.H,v $
 * Revision 1.7  1997/03/03 11:09:40  stig
 * Major update
 *
 * $Id: Tools.H,v 1.7 1997/03/03 11:09:40 stig Exp stig $
 */

#include <string.h>
#include <iostream.h>

#ifdef unix
#include <cgi/sysdefs.H>
#else
#include <cgi/pcdefs.H>
#endif

#ifdef HAVE_DIRENT_H 
#include <dirent.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

// I usually have a couple of defines to ease my work

#ifndef PARSESIZE
#define PARSESIZE 1024
#endif

// Maximum entries I return from the getfiles functions
#ifndef MAXDIRRETURN
#define MAXDIRRETURN 100
#endif

#ifndef OK
#define OK 1
#endif

#ifndef NOTOK
#define NOTOK !OK
#endif

#ifndef DEFAULT
#define DEFAULT 0
#endif

#ifndef TAB
#define TAB 9
#endif

#ifndef ENDLINECHAR
#define ENDLINECHAR 10
#endif

/*
 * I like the flexibility of macros, and this one is a
 * MUST when also converting to PCs. I have yet to be
 * persuaded into believing that inline is so much better.
 */

#ifndef my_cmp
#define my_cmp
#ifdef HAVE_STRCASECMP
#define caselesscmp(arg1, arg2) (strcasecmp((arg1),(arg2)))
#else
#define caselesscmp(arg1, arg2) (stricmp((arg1),(arg2)))
#endif
#endif

/* A class with small procedures I use again and again and again..
 * My guess is that other people may have use of them too, if for
 * nothing else to learn how I do things.
 */

class Toolbox {
public:
  Toolbox();
  Toolbox(const char *workdir);
  Toolbox(const Toolbox &ct);
  Toolbox& operator=(const Toolbox &ct);
  virtual ~Toolbox();

  // For us who likes life random 
  MYINT pseudoinit(MYINT tmp_seed=NULL);
  MYINT pseudorand(void);
  inline MYINT getRandomNumber(void);

  // For the paranoid ones..
  int paranoidsystem(const char *command) const;

  // Can't take it anymore
  void crashAndBurn(const char *bailoutMsg, int crashLevel) const;

  // File and Directory Tools
  int isDirectory(const char *path) const;  // Ripped from Swish by K. Hughes
  int isFile(const char *file) const;       // Ripped from Swish by K. Hughes
  int isLink(const char *file) const;       // Ripped from Swish by K. Hughes
  MYINT getFilesize(const char *file) const; // Ripped from Swish by K. Hughes
  int readinfile(ostream &doc, const char *filename,
		 const char* directory=NULL) const;
  inline char** gethtmlfiles(const char* directory=NULL) const;
  inline int listhtmlfiles(ostream &doc, const char* directory,
		    int (*lister)(ostream &doc, const char *htmlfile)) const;
#ifdef HAVE_DIRENT_H    
  char** getfiles(const char* directory=NULL,
		  const char *fileExtension=NULL) const;
  int listfiles(ostream &doc, const char *directory, const char *fileExtension,
		int (*lister)(ostream &doc, const char *file)) const;
#endif  
  inline const char *getUserPath(const char *user) const;

  // Someone may want to subclass:    
protected:
#ifdef HAVE_DIRENT_H  
  inline int fileselecter(dirent *entry, const char *extension) const;
#endif  
  inline const char *correctdir(const char *dir) const;
  
  char *directory;
  static MYINT seed;
};

inline int
samplelister(ostream &doc, const char *file){
  /*SDS
   * Sample filelister for the methods in the class.  Simple and
   * powerful (req. htmlTool.H being included somewhere)
   */
  
  doc << "<li>" << file << endl;
  return OK;
}

// Our inlines

inline MYINT
Toolbox::getRandomNumber(void){
  return pseudorand();
};

inline char**
Toolbox::gethtmlfiles(const char* directory/*=NULL*/) const{
  /*SDS
   * Pass the message to the generic function
   */
  
#ifdef unix  
  return getfiles(directory,"html");
#else
  return getfiles(directory,"htm");
#endif  
};

inline int
Toolbox::listhtmlfiles(ostream &doc, const char* directory,
			  int (*lister)(ostream &doc, const char *htmlfile)) const{
  /*SDS
   * Pass the message to the generic function
   */
  
#ifdef unix
  return listfiles(doc, directory, "html", lister);
#else
  return listfiles(doc, directory, "htm", lister);
#endif
};

inline const char *
Toolbox::getUserPath(const char *user) const{
  /*SDS
   * If on a unix-system or a system with /etc/passwd it returns the
   * user-path.
   */
  
#ifndef HAVE_PWD_H
  return "Your 'puter does not have a user-system, so no path for you";
#else
  passwd *pwd=getpwnam(user);
  if (!pwd) return NULL;
  else return pwd->pw_dir;
#endif  
};

#ifdef HAVE_DIRENT_H
inline int
Toolbox::fileselecter(dirent *entry, const char *extension) const{
  /*SDS
   * Easy-to-use function if you have a POSIX compatible
   * compiler
   */
  
  if (!extension) return OK;
  if (
      // Uncomment following to restrict files found 
      //      !(strcmp(entry->d_name,"."))||
      //      !(strcmp(entry->d_name,".."))||
      //      (entry->d_name[0]=='.')||
      !strstr(entry->d_name, extension) //Does only patternmatching
      )
        return 0;
  else return OK;
};
#endif

inline const char *
Toolbox::correctdir(const char *dir) const{
  /*SDS
   * Hack to find the correct dir at all times
   */
  
  if (!dir) return directory;
  else return dir;
};

//That's all folks.  

#endif
