#ifndef HTMLTOOL_H_
#define HTMLTOOL_H_ 

/*
 * Definitions for full the html-tool
 * Updated $Date: 1997/03/03 11:07:49 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: htmlTool.H,v $
 * Revision 1.7  1997/03/03 11:07:49  stig
 * updated for v0.2
 *
 *
 * $Id: htmlTool.H,v 1.7 1997/03/03 11:07:49 stig Exp stig $
 */

#include <iostream.h>
#include <cgi/Defs.H>

class htmlTool : public ostream {

protected:
  htmlTool& outputtag(int &type, const char *value);
  htmlTool& outputtag(const char *value, const char *type);
  inline void initvars(void);
  
public:
  htmlTool();
  htmlTool( ostream& dest );
  ~htmlTool( void );
  
  htmlTool& gotoURL( const char *redirectUrl );
  htmlTool& headerStatus(int status, const char *statusMessage);
  htmlTool& printHeader( void );
  htmlTool& printPlainHeader( void );
  htmlTool& headerContentType( const char *mimeType );

  inline htmlTool& html( void );
  inline htmlTool& title( void );
  inline htmlTool& title( const char *content );
  inline htmlTool& head( void );
  htmlTool& body( const char *bodyInfo = NULL );
  inline htmlTool& foot( void );
  inline htmlTool& address( void );
  inline htmlTool& address( const char *content );
  htmlTool& comment( void );
  htmlTool& comment( const char *content );
    
  htmlTool& put( char *text );
  htmlTool& put( const char *text );
  htmlTool& put( int i );
  htmlTool& put( long l );
  htmlTool& put( float f );
  htmlTool& put( double d );
  
  inline htmlTool& ul( void );
  inline htmlTool& ol( void );
  htmlTool& li( void );  inline htmlTool& dl( void );
  htmlTool& dd( void );  htmlTool& dt( void );

  htmlTool& link( const char *location, const char *description );
  htmlTool& image( const char *imageFile, const char *altText = NULL );

  htmlTool& br( void );
  htmlTool& hr( void );
  htmlTool& hr( int percent );
  htmlTool& hr( int percent, int size );

  htmlTool& lf( void );
  inline htmlTool& p( void );
  htmlTool& p1( void );
  
  inline htmlTool& h1( void );  inline htmlTool& h1( const char *content );
  inline htmlTool& h2( void );  inline htmlTool& h2( const char *content );
  inline htmlTool& h3( void );  inline htmlTool& h3( const char *content );
  inline htmlTool& h4( void );  inline htmlTool& h4( const char *content );
  inline htmlTool& h5( void );  inline htmlTool& h5( const char *content );
  inline htmlTool& h6( void );  inline htmlTool& h6( const char *content );
  
  inline htmlTool& b( void );
  inline htmlTool& b( const char *content );
  inline htmlTool& i( void );
  inline htmlTool& i( const char *content );
  inline htmlTool& fixed( void );
  inline htmlTool& fixed( const char *content );
  inline htmlTool& underline( void );
  inline htmlTool& underline( const char *content );
  inline htmlTool& define( void );
  inline htmlTool& define( const char *content );
  inline htmlTool& em( void );
  inline htmlTool& em( const char *content );
  inline htmlTool& cite( void );
  inline htmlTool& cite( const char *content );
  inline htmlTool& code( void );
  inline htmlTool& code( const char *content );
  inline htmlTool& kbd( void );
  inline htmlTool& kbd( const char *content );
  inline htmlTool& samp( void );
  inline htmlTool& samp( const char *content );
  inline htmlTool& var( void );
  inline htmlTool& var( const char *content );
  inline htmlTool& strong( void );
  inline htmlTool& strong( const char *content );
  inline htmlTool& strike( void );
  inline htmlTool& strike( const char *content );
  inline htmlTool& block( void );
  inline htmlTool& block( const char *content );
  inline htmlTool& pre( void );
  inline htmlTool& pre( const char *content );
  inline htmlTool& center( void );
  inline htmlTool& center( const char *content );

  inline htmlTool& big( void );
  inline htmlTool& big( const char *content );
  inline htmlTool& small( void );
  inline htmlTool& small( const char *content );
  inline htmlTool& sub( void );
  inline htmlTool& sub( const char *content );
  inline htmlTool& sup( void );
  inline htmlTool& sup( const char *content );
  inline htmlTool& subscript( void );
  inline htmlTool& subscript( const char *content );
  inline htmlTool& superscript( void );
  inline htmlTool& superscript( const char *content );

  htmlTool& font( char action = '+', int size = 1 );
  
private:
  
  int cgiHTML, cgiTITLE, cgiHEAD, cgiBODY, cgiFOOT, cgiADDRESS;
  int cgiH1, cgiH2, cgiH3, cgiH4, cgiH5, cgiH6, cgiBOLD;
  int cgiITALIC, cgiFIXED, cgiUNDERLINE, cgiDEFINE, cgiEMPHASIS;
  int cgiCITE, cgiCODE, cgiKBD, cgiSAMP, cgiVAR, cgiSTRONG, cgiBLOCK;
  int cgiPRE, cgiCENTER, cgiFONT, cgiBIG, cgiSMALL, cgiSTRIKE;
  int cgiSUBSCRIPT, cgiSUPERSCRIPT, cgiCOMMENT, cgiUL, cgiOL, cgiDL, cgiP;
};

extern ostream& br(ostream& outs);
extern ostream& hr(ostream& outs);
extern ostream& p(ostream& outs);
extern ostream& li(ostream& outs);
extern ostream& dd(ostream& outs);

// Inline Functions

inline void
htmlTool::initvars(void){
  // Ugly as hell :-)
  cgiHTML= cgiTITLE= cgiHEAD= cgiBODY= cgiFOOT= cgiADDRESS= CGIFALSE;
  cgiH1= cgiH2= cgiH3= cgiH4= cgiH5= cgiH6= CGIFALSE;
  cgiBOLD= cgiITALIC= cgiFIXED= cgiUNDERLINE= cgiDEFINE= CGIFALSE;
  cgiEMPHASIS= cgiCITE= cgiCODE= cgiKBD= CGIFALSE;
  cgiSAMP= cgiVAR= cgiSTRONG= cgiBLOCK= cgiPRE= cgiCENTER= CGIFALSE;
  cgiFONT= cgiBIG= cgiSMALL= cgiSUBSCRIPT= CGIFALSE;
  cgiSUPERSCRIPT= cgiSTRIKE= cgiCOMMENT= cgiUL= cgiOL= cgiDL= cgiP= CGIFALSE;
};

inline htmlTool&
htmlTool::html( void ){ return outputtag(cgiHTML, "HTML");}

inline htmlTool&
htmlTool::title( void ){ return outputtag(cgiTITLE, "TITLE");}

inline htmlTool&
htmlTool::title( const char *content ){ return outputtag(content, "TITLE");}

inline htmlTool&
htmlTool::head( void ){ return outputtag(cgiHEAD, "HEAD");}

inline htmlTool&
htmlTool::foot( void ){ return outputtag(cgiFOOT, "FOOT");}

inline htmlTool&
htmlTool::address( void ){ return outputtag(cgiADDRESS, "ADDRESS");}

inline htmlTool&
htmlTool::address( const char *content ){ return outputtag(content, "ADDRESS");}

inline htmlTool&
htmlTool::ul( void ){ return outputtag(cgiUL, "UL"); }

inline htmlTool&
htmlTool::ol( void ){ return outputtag(cgiOL, "OL"); }

inline htmlTool&
htmlTool::dl( void ){ return outputtag(cgiDL, "DL"); }

inline htmlTool&
htmlTool::p( void ){  return outputtag(cgiP, "P"); }

inline htmlTool&
htmlTool::h1( void ){ return outputtag(cgiH1, "H1"); }
inline htmlTool&
htmlTool::h1( const char *content ){ return outputtag(content, "H1"); }
inline htmlTool&
htmlTool::h2( void ){ return outputtag(cgiH2, "H2"); }
inline htmlTool&
htmlTool::h2( const char *content ){ return outputtag(content, "H2"); }
inline htmlTool&
htmlTool::h3( void ){ return outputtag(cgiH3, "H3"); }
inline htmlTool&
htmlTool::h3( const char *content ){ return outputtag(content, "H3"); }
inline htmlTool&
htmlTool::h4( void ){ return outputtag(cgiH4, "H4"); }
inline htmlTool&
htmlTool::h4( const char *content ){ return outputtag(content, "H4"); }
inline htmlTool&
htmlTool::h5( void ){ return outputtag(cgiH5, "H5"); }
inline htmlTool&
htmlTool::h5( const char *content ){ return outputtag(content, "H5"); }
inline htmlTool&
htmlTool::h6( void ){ return outputtag(cgiH6, "H6"); }
inline htmlTool&
htmlTool::h6( const char *content ){ return outputtag(content, "H6"); }

inline htmlTool&
htmlTool::b( void ){ return outputtag(cgiBOLD, "B"); }
inline htmlTool&
htmlTool::b( const char *content ){ return outputtag(content, "B"); }

inline htmlTool&
htmlTool::i( void ){ return outputtag(cgiITALIC, "I"); }
inline htmlTool&
htmlTool::i( const char *content ){ return outputtag(content, "I"); }

inline htmlTool&
htmlTool::fixed( void ){ return outputtag(cgiFIXED, "TT"); }
inline htmlTool&
htmlTool::fixed( const char *content ){ return outputtag(content, "TT"); }

inline htmlTool&
htmlTool::underline( void ){ return outputtag(cgiUNDERLINE, "U"); }
inline htmlTool&
htmlTool::underline( const char *content ){ return outputtag(content, "U"); }

inline htmlTool&
htmlTool::define( void ){ return outputtag(cgiDEFINE, "DFN"); }
inline htmlTool&
htmlTool::define( const char *content ){ return outputtag(content, "DFN"); }

inline htmlTool&
htmlTool::em( void ){ return outputtag(cgiEMPHASIS, "EM"); }
inline htmlTool&
htmlTool::em( const char *content ){ return outputtag(content, "EM"); }

inline htmlTool&
htmlTool::cite( void ){ return outputtag(cgiCITE, "CITE"); }
inline htmlTool&
htmlTool::cite( const char *content ){ return outputtag(content, "CITE"); }

inline htmlTool&
htmlTool::code( void ){ return outputtag(cgiCODE, "CODE"); }
inline htmlTool&
htmlTool::code( const char *content ){ return outputtag(content, "CODE"); }

inline htmlTool&
htmlTool::kbd( void ){ return outputtag(cgiKBD, "KBD"); }
inline htmlTool&
htmlTool::kbd( const char *content ){ return outputtag(content, "KBD"); }

inline htmlTool&
htmlTool::samp( void ){ return outputtag(cgiSAMP, "SAMP"); }
inline htmlTool&
htmlTool::samp( const char *content ){ return outputtag(content, "SAMP"); }

inline htmlTool&
htmlTool::var( void ){ return outputtag(cgiVAR, "VAR"); }
inline htmlTool&
htmlTool::var( const char *content ){ return outputtag(content, "VAR"); }

inline htmlTool&
htmlTool::strong( void ){ return outputtag(cgiSTRONG, "STRONG"); }
inline htmlTool&
htmlTool::strong( const char *content ){ return outputtag(content, "STRONG"); }

inline htmlTool&
htmlTool::strike( void ){ return outputtag(cgiSTRIKE, "STRIKE"); }
inline htmlTool&
htmlTool::strike( const char *content ){ return outputtag(content, "STRIKE"); }

inline htmlTool&
htmlTool::block( void ){ return outputtag(cgiBLOCK, "BLOCKQUOTE"); }
inline htmlTool&
htmlTool::block( const char *content ){ return outputtag(content, "BLOCKQUOTE"); }

inline htmlTool&
htmlTool::pre( void ){ return outputtag(cgiPRE, "PRE");}
inline htmlTool&
htmlTool::pre( const char *content ){ return outputtag(content, "PRE");}

inline htmlTool&
htmlTool::center( void ){ return outputtag(cgiCENTER, "CENTER"); }
inline htmlTool&
htmlTool::center( const char *content ){ return outputtag(content, "CENTER"); }

inline htmlTool&
htmlTool::big( void ){ return outputtag(cgiBIG, "BIG");}
inline htmlTool&
htmlTool::big( const char *content ){ return outputtag(content, "BIG");}

inline htmlTool&
htmlTool::small( void ){ return outputtag(cgiSMALL, "SMALL");}
inline htmlTool&
htmlTool::small( const char *content ){ return outputtag(content, "SMALL");}

inline htmlTool&
htmlTool::sub( void ){ return outputtag(cgiSUBSCRIPT, "SUB");}
inline htmlTool&
htmlTool::sub( const char *content ){ return outputtag(content, "SUB");}

inline htmlTool&
htmlTool::sup( void ){ return outputtag(cgiSUPERSCRIPT, "SUP");}
inline htmlTool&
htmlTool::sup( const char *content ){ return outputtag(content, "SUP");}

inline htmlTool&
htmlTool::subscript( void ){ return outputtag(cgiSUBSCRIPT, "SUB");}
inline htmlTool&
htmlTool::subscript( const char *content ){ return outputtag(content, "SUB");}

inline htmlTool&
htmlTool::superscript( void ){ return outputtag(cgiSUPERSCRIPT, "SUP");}
inline htmlTool&
htmlTool::superscript( const char *content ){ return outputtag(content, "SUP");}


#endif


