/*
 * Implementation of class CgiData
 * Updated $Date: 1997/09/08 08:35:48 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: CgiData.cpp,v $
 * Revision 1.8  1997/09/08 08:35:48  stig
 * done nothing I guess
 *
 * Revision 1.7  1997/03/03 11:10:36  stig
 * Major update
 *
 *
 * $Id: CgiData.cpp,v 1.8 1997/09/08 08:35:48 stig Exp $
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <cgi/CgiData.H>

 
CgiData::CgiData( ) {

  /*SDS
   * The basic constructor sets the debugging options and
   * creates a CgiEnv object whereafter it reads in the data
   * as POST or GET.
   */
  
  itsFirstEntry=NULL;
  itsEnv = new CgiEnv();

  if (MAINT) DEBUGPUT("Creating the Cgi Object");
  if ( noCaseCompare( itsEnv->getReqMethod(), "post" ) )
    postFormInput();
  else
    getFormInput();
};


CgiData::~CgiData( void ){
  /*SDS
   * Deletes the CgiEnv object
   */
  if (MAINT) DEBUGPUT("Destructing the data: ");
  if (itsFirstEntry) itsFirstEntry->dumpall();
  
  if (itsFirstEntry) delete itsFirstEntry;
  delete itsEnv;
};

int
CgiData::parseFormInput( const char *data, int length ){

  /*SDS
   * Parses all the input in <i>data</i> and stores it in the
   * list.
   */
  DEBUGPUT("Parsing: " << data << "$$" << length);
  int pos = 0;
  FormEntry *n;
  FormEntry *l = NULL;

  while ( pos != length ) { 
    int foundEq = 0, foundAmp = 0, len = 0;
    int start = pos;
    char *attr, *value;
    
    while ( pos != length ) { 
      if ( data[pos] == '=' ){
	foundEq = 1;
	pos++; break;
      }
      pos++; len++;
    }
      
    if (!foundEq){break;}
      
    if ( ! unEscapeChars( &attr, data+start, len ) ) 
      _CgiError("Insuff. Mem. (CgiData::parseFormInput)");
      
    start = pos;
    len = 0;
    while (pos != length) { /* pos != length */
      if (data[pos] == '&'){
	foundAmp = 1;
	pos++; break;
      }
      pos++; len++;
    }
      
    /* The last pair probably won't be followed by a &, but
       that's fine, so check for that after accepting it */
    if ( ! unEscapeChars( &value, data+start, len ) ) 
      _CgiError("Insufficient Memory (CgiData::parseFormInput)");
      
    /* OK, we have a new pair, add it to the list. */
    n = new FormEntry(attr, value);	
    
    if ( ! l ) itsFirstEntry = n;
    else l->setNext(n);
      
    l = n;
    if ( ! foundAmp )	break;
  }
  
  return CGIOK;
}


FormEntry*
CgiData::findNextEntry(const char *findTarget, FormEntry *findPos) const {
  /*SDS
   * Functions who finds the next entry from position <i>findPos</i>
   * with name <i>findTarget</i>
   */
  //  if (MAINT) DEBUGPUT("Searching for " << findTarget);
  FormEntry *c = findPos;
  
  while (c){
    if (!strcmp(c->Attr(), findTarget))      return c;
    c = c->Next();
  }
  //  if (MAINT) DEBUGPUT("Did not find " << findTarget);
  return NULL;
};

char* 
CgiData::getvar( char *name, int max, int& found, int newlines/*=1*/ ) {
  /*SDS
   * Fetches the variable named <i>name</i>
   */

  if (!name) return 0L;
  FormEntry *e = findEntry( name );

  if ( ! e ){
    found = CGIFALSE;
    return 0L;
  }
  
  char *temp = new char[strlen(e->Value())+1];

  if ( ! temp ) _CgiError( "Insufficient Memory (CgiData::fetchString)" );

  if ( ! makeString( e, temp, max, newlines ) ) {
    found = CGITRUE;
    return 0L;
  }
  found = CGITRUE;
  
  return temp;
};

int 
CgiData::getint( char *name, int& found ){
  /*SDS
   * Fetches an integer variable named <i>name</i>
   * The method uses atoi() in stdlib.h
   * Returns 0 if something wrong happened. (if that causes problems,
   * fetch it as a string and handle it yourself)
   */

  FormEntry *e = findEntry( name );  
  if ( ! e || ! strlen( e->Value() ) ) {
    found = CGIFALSE;
    return 0;
  }	
  int ch = firstNonspaceChar( e->Value() );
  if ( ! ( isdigit( ch ) ) && ( ch != '-' ) && ( ch != '+' ) ) {
    found = CGIFALSE;
    return 0;
  } 
  else {
    found = CGITRUE;
    int temp = atoi( e->Value() );
    return temp;
  }
};

double
CgiData::getdouble( char *name, int& found ){
  /*SDS
   * Fetches a double variable named <i>name</i>
   * The method uses atof() in stdlib.h
   * Returns 0 if something wrong happened. (if that causes problems,
   * fetch it as a string and handle it yourself)
   */
  
  FormEntry *e = findEntry( name );
  if ( ! e || ! strlen( e->Value() ) ) {
    found = CGIFALSE;
    return 0;
  } 
  int ch = firstNonspaceChar( e->Value() );
  if ( ! isdigit( ch) && ch != '.' && ch != '-' && ch != '+' ) {
    found = CGIFALSE;
    return 0;
  } 
  else {
    found = CGITRUE;
    double temp = atof( e->Value() );
    return temp;
  }
}

char** 
CgiData::getmultivar( char *name, int& numFound ){
  /*SDS
   * Fetches several strings with the same variable name
   */
  
  int index = 0;
  char **result = 0;
  
  FormEntry *e = findEntry( name );
  
  if ( ! e ) { numFound = 0; return NULL;  }
  
  do {

    if ( ! result ){
      result = new char*[index + 1];
      if ( ! result )_CgiError("Insufficient Memory (CgiData::fetchSelectMultiple");
    }
    else {	
      char** temp = new char*[ index + 1 ];       // get new space
      
      if(!temp) _CgiError("Insufficient Memory (CgiData::fetchSelectMultiple." );
      
      // copy old into new, minus next element
      for ( int i = 0; i < index; i++ ){
	temp[i] = new char[ strlen( result[i] ) + 1 ];
	strcpy( temp[i], result[i] );
      }
      
      if (result) delete [] result;           // delete old space
      result = temp;                          // install new
      
    }
    
    result[index] = new char[ strlen(e->Value()) + 1 ];
    if ( ! result[index] )
      _CgiError("Insuff Mem (CgiData::fetchSelectMultiple." );
    
    strcpy( result[index], e->Value() );
    index++;

    e=findNextEntry(name, e->Next());

  }
  while (e);
  
  numFound = index;
  return result;
}

int 
CgiData::makeString( FormEntry *e, char *result, int max, int newlines) const {
  /*SDS
   * Creates a string to fit the FormEntry.
   */
  
  char *dp;
  int truncated = 0, len = 0, avail = max-1;
  int crCount = 0, lfCount = 0;	
  
  dp = result;
  const char *sp = e->Value();	
  
  while (1) {
    int ch = *sp;
    if ( len == avail ) {truncated = 1;break;}	 
    /* Fix the CR/LF, LF, CR nightmare: watch for
     * consecutive bursts of CRs and LFs in whatever
     * pattern, then actually output the larger number
     * of LFs. Consistently sane, yet it still allows
     * consecutive blank lines when the user actually
     * intends them.
     */
    if ( ( ch == 13 ) || ( ch == 10 ) ) 
      if ( ch == 13 )  crCount++;
      else lfCount++;
    else {
      if ( crCount || lfCount ) {
	int lfsAdd = crCount;
	if ( lfCount > crCount ) 
	  lfsAdd = lfCount; 
	/* Stomp all newlines if desired */
	if ( ! newlines ) lfsAdd = 0;
	while ( lfsAdd ) {
	  if ( len == avail ){ truncated = 1; break; }
	  *dp = 10;
	  dp++; lfsAdd--; len++;
	}
	
	crCount = lfCount = 0;
      }
      if ( ch == '\0' ){
	/* The end of the source string */
	break;				
      }	
      *dp = ch;
      dp++; len++;
    }
    sp++;	
  }	
  *dp = '\0';
  
  if ( truncated ) return -1;
  else if (!len) return 0;
  else return 1;
};


int 
CgiData::unEscapeChars( char **sp, const char *cp, int len ) const {
  /*SDS
   * Unescapes the chars in sp
   */

  enum EscapeType { Rest = 0, First, Second };
      
  char 		hexValues[ 256 ];
  char 		*s;
  EscapeType 	escapeState = Rest;
  int 		escapedValue=0, srcPos=0, dstPos = 0;

  int i;
  for ( i = 0; i < 256; i++ ) hexValues[i] = 0;
 
  hexValues['0'] = 0;	  hexValues['1'] = 1;	
  hexValues['2'] = 2;	  hexValues['3'] = 3;	
  hexValues['4'] = 4;	  hexValues['5'] = 5;	
  hexValues['6'] = 6;	  hexValues['7'] = 7;	
  hexValues['8'] = 8;	  hexValues['9'] = 9;
  hexValues['A'] = 10;    hexValues['B'] = 11;
  hexValues['C'] = 12;    hexValues['D'] = 13;
  hexValues['E'] = 14;    hexValues['F'] = 15;
  hexValues['a'] = 10;    hexValues['b'] = 11;
  hexValues['c'] = 12;    hexValues['d'] = 13;
  hexValues['e'] = 14;    hexValues['f'] = 15;

  s = new char[ len + 1 ];
  
  if ( ! s ) _CgiError("Insufficient Memory (unescapeChars)");
  
  while ( srcPos < len ) {
    int ch = cp[ srcPos ];
    
    switch ( escapeState ){
    case Rest:
      if ( ch == '%' ) escapeState = First;
      else if ( ch == '+' ) s[ dstPos++ ] = ' ';
      else s[ dstPos++ ] = ch;	
      break;
    case First:
      escapedValue = hexValues[ ch ] << 4;	
      escapeState = Second;
      break;
    case Second:
      escapedValue += hexValues[ ch ];
      s[ dstPos++ ] = escapedValue;
      escapeState = Rest;
      break;
    }
    srcPos++;
  }
  s[ dstPos ] = '\0';
  *sp = s;
  return CGIOK;
};


int 
CgiData::firstNonspaceChar( const char *s ) const {
  /*SDS
   * Returns the first nonspace char from <i>s</i>
   */
  
  int len = strspn( s, " \n\r\t" );
  return s[len];
};

/* End of class CgiData */
