/*
 * Error Handling and Debugging macros implementation
 *
 * $Log: Debug.cpp,v $
 * Revision 1.5  1997/09/08 08:49:55  stig
 * Updated for v0.3
 *
 * Revision 1.4  1997/03/03 11:12:07  stig
 * Major update
 *
 *
 * $Id: Debug.cpp,v 1.5 1997/09/08 08:49:55 stig Exp $
 */

#include <fstream.h>
#include <time.h>
#include <stdlib.h>
#include <cgi/htmlTool.H>

DebugTool::DebugTool(void){
  /*SDS
   * Opens logFile which should be known by now
   */
  
  open(logFile, ios::app);
};

DebugTool::~DebugTool(void){ // Just flushes
  /*SDS
   * Flushes and bets on the ofstream destructor will
   * deal with the rest
   */
  
  flush();
};

int
cgiError(const char *msg, int line, const char *err_file){
  /*SDS
   * Error function. Should be called through the macro
   * <i>_CgiError(arg)</i> where arg is a const char *msg
   * I know macros are non-thrustworthy, but here it is
   * necessary.
   */
  
  ofstream file( logFile, ios::app );
  
  htmlTool	errDoc( cout );
  time_t 	Now;
  tm 		*Date;
  char 		S[80];
  
#ifdef CGIPPDEBUG 
  file << "#### error condition encountered" << endl
       << "message = " << msg << endl;
#endif
  
  Now = time( NULL );
  Date = localtime( &Now );
  strftime( S, 80, "%A, %B %d, %Y %I:%M:%S %p", Date );
  
  errDoc.printHeader().html().head().title().put( "Cgipp Error: " );
  errDoc.put( msg ).title().head().lf().br().body();
  errDoc.h1().put( "Cgipp Error. " ).br().h1();
  errDoc.font('-',2 ).put( S ).font();
  errDoc.p().p().h2().put( "Specification: " );
  errDoc.put( msg ).h2();
  errDoc.br().p().p().lf();
  errDoc << "Cgipp encountered an error while executing the "
	 << "cgi code. If the error is due to low memory or "
	 << "an input/output difficulty, please try re-calling "
	 << "the program.  Otherwise, contact the owner of "
	 << "the script, the Cgipp-Admin (" << admin
	 << ", or the webmaster at the site for assistance. "
	 << "Please include linenumber: " << line << " and "
	 << "the filename: " << err_file << br << p;
  errDoc.body().html();
    
  exit(-1);
  return -1;
};

