/*
 * CGI Toolbox - Sourcefile
 *
 * $Log: Tools.cpp,v $
 * Revision 1.8  1997/09/08 08:52:45  stig
 * Updated for v0.3
 *
 * Revision 1.7  1997/03/03 11:12:48  stig
 * Major update
 *
 * $Id: Tools.cpp,v 1.8 1997/09/08 08:52:45 stig Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <sys/stat.h>
#include <cgi/Tools.H>

// For our small neat random things:
#ifndef unix
#define RAND_A  16807L       /* 7^5      */
#define RAND_M  2147483647L  /* Mersenne prime 2^31 - 1 */
#define RAND_Q  127773L      /* m/a      */
#define RAND_R  2836L        /* m%a      */
#else
#define RAND_A  16807       /* 7^5      */
#define RAND_M  2147483647  /* Mersenne prime 2^31 - 1 */
#define RAND_Q  127773      /* m/a      */
#define RAND_R  2836        /* m%a      */
#endif

MYINT Toolbox::seed=0;

//Here we go...

Toolbox::Toolbox(){
  /*SDS
   * Basic Constructor
   */
  
  directory = new char[2];
  strcpy(directory,".");
};

Toolbox::Toolbox(const char *workdir){
  /*SDS
   * Constructor who sets the workdirectory
   */
  
  if (isDirectory(workdir)){
    directory = new char[strlen(workdir)+1];
    strcpy(directory, workdir);
  }
  else {
    directory = new char[2];
    strcpy(directory,".");
  }
};

Toolbox::Toolbox(const Toolbox &ct){
  /*SDS
   * Calls the assignment operator
   */
  
  *this=ct;
};

Toolbox&
Toolbox::operator=(const Toolbox &ct){
  /*SDS
   * Assignment operator
   */
  
  if (directory) delete [] directory;
  directory = new char[strlen(ct.directory)+1];
  strcpy(directory, ct.directory);
  return *this;
};
  
Toolbox::~Toolbox(){
  /*SDS
   * Deletes allocated memory
   */
  
  if (directory) delete [] directory;
};

MYINT
Toolbox::pseudoinit(MYINT tmp_seed/*=NULL*/){
  /*SDS
   * Initializes the random-functions. if tmp_seed is not
   * given, one is chosen dependent on the OS
   */
  
  if (!tmp_seed)
#ifdef unix
    tmp_seed = ((MYINT) getpid() << 16) ^ time(NULL);
#else
    tmp_seed = ((MYINT)  clock() << 16) ^ time(NULL);
#endif
  seed = tmp_seed;
  return  seed;
};

MYINT
Toolbox::pseudorand(void){
  /*SDS
   * Picks a random number
   */
  
  MYINT tmp_seed = RAND_A * (seed % RAND_Q) - RAND_R * (seed / RAND_Q);
  if (tmp_seed >= 0) seed = tmp_seed;
  else seed = tmp_seed + RAND_M;
  return seed;
};


int
Toolbox::paranoidsystem(const char *command) const{
  /*SDS
   * A pretty safe way to do system-calls. Check the code
   * if you want to tune it more for your use.
   */
  
  int len = strlen(command);
  char *str = new char[len+1];
  char c;
  for (int i=0;i<len;i++){
    c=str[i];
    if (isalnum(c)) ;
    else str[i]='_';
  }
  len = system(str);
  delete[] str;

  return len;
};

void
Toolbox::crashAndBurn(const char *bailoutMsg, int crashLevel) const{
  /*SDS
   * A rudimentary Crash-and-Burn message which exits
   * with crashLevel
   */
  
  cerr << "<BLINK><font size=\"+7\" color=\"red\">"
       << bailoutMsg << "</font></BLINK>" << endl;
  exit(crashLevel);
};

int
Toolbox::readinfile(ostream &doc, const char *filename,
		       const char* directory/*=NULL*/) const{
  /*SDS
   * Reads in chosen file from given directory to doc.
   * If directory is not given, it is read from the
   * current directory
   */

  const char *dirtoopen = correctdir(directory);

  if (!isDirectory(dirtoopen)) return NOTOK;
  
  if (filename&&doc.good()){
    char temp[PARSESIZE];
    sprintf(temp, "%s%s", dirtoopen, filename);
    FILE *filetoread = fopen(temp,"r");
    if (filetoread)
      while(fgets(temp, PARSESIZE-1,filetoread))
	doc << temp;
  }
  else return NOTOK;
  return OK;
};

#ifdef HAVE_DIRENT_H
char**
Toolbox::getfiles(const char *directory/*=NULL*/,
		  const char *fileExtension/*=NULL*/) const{
  /*SDS
   * Fetches an array of the files with chosen fileExtension
   */

  char *mydirentries[MAXDIRRETURN]; int i=0;

  const char *dirtoopen = correctdir(directory);
  
  if (!isDirectory(dirtoopen)) return mydirentries;
  
  dirent *direntp;
  DIR *dirp = opendir( dirtoopen );

  for(;(direntp = readdir( dirp ));i++){
    if (fileselecter(direntp, fileExtension)){
      mydirentries[i] = new char[strlen(direntp->d_name)+1];
      strcpy(mydirentries[i], direntp->d_name);
    }
  }
  mydirentries[i] = NULL;
  
  closedir( dirp );

  return mydirentries; 
};

int
Toolbox::listfiles(ostream &doc, const char *directory,
		   const char *fileExtension,
		   int (*lister)(ostream &doc, const char *file))
     const{
  /*SDS
   * The function lists the files in the given directory
   * where each file is sent to the lister-function given
   * as the last argument
   */
  
  const char *dirtoopen = correctdir(directory);
  if (!isDirectory(dirtoopen)) return NOTOK;
  dirent *direntp;
  DIR *dirp = opendir( dirtoopen );

  while ( (direntp = readdir( dirp )) )
    if (fileselecter(direntp, fileExtension))
      lister(doc, direntp->d_name);
  
  closedir( dirp );

  return OK;
};
#endif

/*SDS
 * The following methods have been blatantly borrowed from
 * SWISH by Kevin Hughes. I doubt he'll be extremely mad at
 * me for borrowing his eminent code, so I am cheeky enough
 * to include it. Thanks to you Kevin! :-)
 */
 
int
Toolbox::isDirectory(const char *path) const {
  /*SDS
   * Checks if the <i>path</i> is indeed a directory
   */
  
  struct stat stbuf;

  if (stat(path, &stbuf)) return 0;
  return ((stbuf.st_mode & S_IFMT) == S_IFDIR) ? 1 : 0;
}

int
Toolbox::isFile(const char *file) const {
  /*SDS
   * Checks if the <i>file</i> is indeed a file
   */
   
  struct stat stbuf;

  if (stat(file, &stbuf)) return 0;
  return ((stbuf.st_mode & S_IFMT) == S_IFREG) ? 1 : 0;
}

int
Toolbox::isLink(const char *file) const {
  /*SDS
   * Checks if the <i>file</i> is a symbolic/hard link
   * always returns 0 if on non-unix system
   */
  
#ifndef unix // symbolic links are unix, or?
  return 0;
#else  
  struct stat stbuf;
  
  if (lstat(file, &stbuf)) return 0;
  return ((stbuf.st_mode & S_IFLNK) == S_IFLNK) ? 1 : 0;
#endif  
}

MYINT
Toolbox::getFilesize(const char *file) const {
  /*SDS
   * Returns the filesize of <i>file</i>
   */
  
  struct stat stbuf;
	
  if (stat(file, &stbuf)) return -1;
  return stbuf.st_size;
};

/* End Toolbox */
