/*
 * Implementation of class htmlTool
 * Updated $Date: 1997/09/08 08:54:32 $ by Stig Erik Sand <stig@ii.uib.no>
 *
 * $Log: htmlTool.cpp,v $
 * Revision 1.11  1997/09/08 08:54:32  stig
 * updated for v0.3
 *
 * Revision 1.10  1997/03/03 11:12:39  stig
 * Major update
 *
 *
 * $Id: htmlTool.cpp,v 1.11 1997/09/08 08:54:32 stig Exp $
 */

#include <cgi/htmlTool.H>
#include <string.h>

htmlTool::htmlTool(){
  /*SDS
   * Initialises htmlTool to use stdout for output
   */
  
  init(cout.rdbuf());
  initvars();
};

htmlTool::htmlTool( ostream& dest ){
  /*SDS
   * Initialises htmlTool with <i>dest</i> for output
   */
  
  init(dest.rdbuf());
  initvars();
};

htmlTool::~htmlTool( void ){
  /*SDS
   * Flushes the output and gambles on the superclass destructor to
   * do the rest of the job
   */
  
  flush();
}

htmlTool& 
htmlTool::gotoURL( const char *redirectUrl ) {
  /*SDS
   * Sends a HTTP redirect. Check basic HTTP documentation
   * for how to use it properly
   */
  
  *this << "Location: " << redirectUrl << endl << endl;
  return *this;
}

htmlTool& 
htmlTool::headerStatus(int status, const char *statusMessage ) {
  /*SDS
   * Sends a status message. Check HTTP docs
   */
  
  *this << "Status: " << status << " " << statusMessage
	<< "\n" << endl;
  return *this;
};

htmlTool&
htmlTool::printHeader( void ){
  /*SDS
   * Prints the standard html-header which is needed for
   * a basic document
   */
  
  *this << "Content-type: text/html\n" << endl; return *this;
};

htmlTool&
htmlTool::printPlainHeader( void ){
  /*SDS
   * Print a plain text header
   */
  
  *this << "Content-type: text/plain\n" << endl; return *this;
}

htmlTool& 
htmlTool::headerContentType( const char *mimeType ) {
  /*SDS
   * Ceneric function which outputs a line with the specified
   * MIME-type (mimeType).
   */
  
  *this << "Content-type: " << mimeType << endl << endl;
  return *this;
}

htmlTool&
htmlTool::outputtag(int &type, const char *value){
  /*SDS
   * Generic function for outputting html/sgml tags from
   * htmlTool
   */
  
  if (type==CGIFALSE) {
    type= CGITRUE; *this << "<" << value << ">"; }
  else {
    type=CGIFALSE; *this << "</" << value << ">"; }
  return *this;
};

htmlTool&
htmlTool::outputtag(const char *value, const char *type){
  /*SDS
   * Simple and generic function for easy outputting of
   * html/sgml tags
   */
  
  *this << "<" << type << ">" << value << "</" << type << ">"; 
  return *this;
};

htmlTool&
htmlTool::body( const char *bodyInfo ){
  /*SDS
   * Prints a body tag with bodyInfo as in:<br>
   * &lt;BODY <i>bodyInfo</i>&gt;
   */
  
  if ( cgiBODY == CGIFALSE ) {
    *this << "<BODY";
    if(bodyInfo&&strlen(bodyInfo)) *this << " " << bodyInfo;
    *this <<  ">"; cgiBODY = CGITRUE;
  }
  else {
    *this << "</BODY>"; cgiBODY = CGIFALSE; }
  return *this;
}

htmlTool&
htmlTool::comment( void ){
  /*SDS
   * Prints beg./end of an HTML-comment (equal to a SGML-comment)
   */
  
  if ( cgiCOMMENT == CGIFALSE ){
    *this << "<!-- "; cgiCOMMENT = CGITRUE; }
  else {
    *this << " -->"; cgiCOMMENT = CGIFALSE; }
  return *this;
}

htmlTool&
htmlTool::comment( const char *content ){
  /*SDS
   * Prints content in an HTML/SGML-comment
   */
  
  *this << "<!-- " << content << " -->";
  return *this;
}

htmlTool&
htmlTool::put( char *text ){ *this << text; return *this; }

htmlTool&
htmlTool::put( const char *text ){ *this << text; return *this; }

htmlTool&
htmlTool::put( int i ){ *this << i; return *this; }

htmlTool&
htmlTool::put( long l ){ *this << l; return *this; }

htmlTool&
htmlTool::put( float f ){ *this << f; return *this; }

htmlTool&
htmlTool::put( double d ){ *this << d; return *this; }

htmlTool&
htmlTool::li( void ){  *this << "<LI>"; return *this; }

htmlTool&
htmlTool::dd( void ){ *this << "<DD>"; return *this; }

htmlTool&
htmlTool::dt( void ){ *this << "<DT>"; return *this; }

htmlTool&
htmlTool::link( const char *location, const char *description ) {
  /*SDS
   * Prints an html-link/anchor to <i>location</i> and with
   * description, as in:<br>
   * &lt;A HREF="<i>location</i>"&gt;description&lt;/a&gt;
   */
  
  *this << "<A HREF=\"" << location << "\">" << description << "</A>";
  return *this;
}

htmlTool&
htmlTool::image( const char *imageFile, const char *altText ){
  /*SDS
   * Prints an image-reference to <i>imageFile</i> with alternative
   * text <i>altText</i>
   */
  
  *this << "<IMG SRC=\"" << imageFile << "\" ALT=\"" << altText << "\"";
  return *this;
}

htmlTool&
htmlTool::br( void ){ *this << "<BR>"; return *this; }

ostream&
br(ostream& outs){ outs << "<BR>" << endl; return outs;}

ostream&
hr(ostream& outs){ outs << endl << "<HR>" << endl; return outs;}

ostream&
li(ostream& outs){ outs << "<LI>"; return outs;}

ostream&
p(ostream& outs){ outs << "<P>" << endl; return outs;}

ostream&
dd(ostream& outs){ outs << "<DD>"; return outs;}

htmlTool&
htmlTool::hr( void ){ *this << "<HR>" << endl; return *this; }

htmlTool&
htmlTool::hr( int percent ){
  *this << "<HR WIDTH=\"" << percent << "%\">" << endl;
  return *this;
};

htmlTool&
htmlTool::hr( int percent, int size ){
  /*SDS
   * Prints a ruler with given percent-width and size
   */
  
  *this << "<HR WIDTH=\"" << percent << "%\" SIZE=\""
	<< size << "\">" << endl;
  return *this; }

htmlTool&
htmlTool::lf( void ){
  *this << endl; return *this; }

htmlTool&
htmlTool::p1( void ) { *this << "<P>"; return *this;}

htmlTool&
htmlTool::font( char action, int size ){
  /*SDS
   * Prints a font tag (also end-tag) as in:<br>
   * &lt;FONT SIZE="<i>action</i> <i>size</i>"&gt;
   */
   
  if ( cgiFONT == CGIFALSE ) {
    *this << "<FONT SIZE=" << action << size << ">";
    cgiFONT = CGITRUE;
  }
  else {
    *this << "</FONT>";
    cgiFONT = CGIFALSE;
  }
  return *this;
};

/* Thanks for all the fish */

