/*
 * test.cpp
 *
 * Example program of capabilites of Cgicc by Stephen F. Booth
 *
 * Changed and Improved for Cgipp by Stig Erik Sand
 *
 * $Id: test.cpp,v 1.6 1997/03/03 11:13:01 stig Exp stig $
 *
 */

#include <stdio.h>
#include <limits.h>
#include <cgi/Cgipp.H>

// Uncomment to include Toolbox
#include <cgi/Tools.H>

int
main(int/* argc*/, char ** /*argv*/){ // to avoid warnings

  Cgipp *cgi = new Cgipp();
  
  // Checking debugging or not debugging
  
  int debugging;
  if (cgidebug.good()) debugging = CGITRUE;
  else debugging = CGIFALSE;
  
  // Doing a check whether we have write access or not.
  int writeaccess=0;

  ofstream testfile("out_test",ios::out);
  if (testfile.good()){
    writeaccess=1;
    remove("out_test"); //Deleting our testfile if we had axx
  }
  if (debugging == CGITRUE)
    writeaccess ?
      cgidebug << "Writeaccess" << endl :
      cgidebug << "no Writeaccess" << endl;
  
  if (debugging == CGITRUE)
    cgidebug << "Just going through main()" << endl;

  // Creating an HTMLDocument to out as specified in function header
  htmlTool doc(cout);

  if (debugging == CGITRUE)
    cgidebug << "Instansiated htmlTool doc" << endl;

  //Print "header" for document

  doc.printHeader().html().head().title("Cgipp Test Document").head().lf();
  doc.body("bgcolor=\"#ffffff\"").h1("Cgipp Test Document").lf();

  doc << dd << "First we'll test if HTMLDocument works properly as an \n"
      << "ostream and a few small finesses. To get the full feel of \n"
      << "the example, check the source code (test.cpp) and the source \n"
      << "of the document you get/read now." << br << br << p;

  doc << li << "list item one" << endl;
  doc << li << "list item two" << endl;
  doc << li << "..." << endl;
  doc << li << "list item n" << endl;

  doc << hr;

  doc << dd << "Now we should test some formatting in different \n"
      << "ways. Refer to the documentation for the full list of \n"
      << "available member functions." << p;

  doc.comment("This is a comment followed by linefeed").lf();
  doc.address("My address is stig@ii.uib.no").lf();
  doc.link("http://www.ii.uib.no/~stig/",
	   "This is a link to my home page").lf();
  doc.br();
  doc.h1().put("This is a _large_ header").h1().lf();
  doc.h1("This is also a _large_ header").lf();
  doc.put("This text is: ").b("bold ").i("italic").br().lf();
  doc.put("This text is: ");
  doc.b().put("bold ").i().put("bold-italic").i().b().br().lf();
  doc.hr(60);

  doc << dd << "The same applies for most of the other formatting, \n"
      << "and should be relatively easy to use." << p;

  doc << dd << "Now we should check our neat toolbox before we venture \n"
      << "down the Alley of Form Retrieval. The toolbox provides several \n"
      << "easy-to-use functions/methods/procedures/(whatever name you \n"
      << "use) which will ease most of your work, because you are likely \n"
      << "to use them frequently." << p;

  /*
   * First we should get ourselves an CgiToolbox
   * The directory we give may not be valid on all platforms
   * but . usually is the current dir.
   */
#ifdef TOOLBOX_H_
    Toolbox tools(".");

  // Now let us list all files in our directory
    doc.h2("The files in the current directory").lf();
    tools.listfiles(doc, ".", NULL, samplelister);
    doc << hr << p;
#endif
    
  doc << "For new users of the neat tracing macros, we have this \n"
      << "neat example which first prints the command and then \n"
      << "executes it. (The &lt;&lt; may crash some browsers)" << br;
  _TRACE(doc, doc << "This is within the trace." << br;);

  // Let us delve into the formentries now..
  
  doc << "Let us get the name and the other variables now..." << p;

  if (debugging == CGITRUE)
    cgidebug << "Getting variables" << endl;
  
  int found; 
 
  char *name = cgi->getvar("name", 81, found, 0);
  
  if ( ! found || ! name )
    doc << "Name = [none entered]" <<  br;
  else {
    doc << "Name = " << name <<  br;
    delete [] name; // Remember to delete
  }
  
  char *address = cgi->getvar("address", 241, found );
  if ( ! found || ! address )
    doc << "Address not found." << br;
  else {
    doc.put("Address: ").pre().put(address).pre().br().lf();
    delete [] address;
  }
  
  if ( cgi->getvar("hungry", 5, found ) ) // Memory leak
    doc << "I'm Hungry!" << br;
  else
    doc << "I'm not Hungry!" << br;
  
  double temperature = 
    cgi->getdouble("temperature", found);
  if ( ! found )
    doc << "Temperature not found" << br;
  else
    doc << "My temperature is " << temperature << " degrees" << br;
  
  int frogsEaten = cgi->getint("frogs", found );
  if ( ! found )
    doc << "Variable frogsEaten not found" << br;
  else
    doc << "I have eaten " << frogsEaten << " frogs" << br;
  
  char *colorChoice = cgi->getvar("colors", 20, found );
  if ( ! found )
      doc << "Colors not found" << br;
  else
      doc << "I am: " << colorChoice << br;

  // I put _i_ here instead of in each for-loop, to let archaic
  // C++ compilers also compile it properly
  int i=0, num=0; 
    
  char **res = cgi->getmultivar( "flavors", num );
  
  if ( ! res ) 
    doc << "I hate ice cream." << br;
  else {
    doc.h4("My favourite ice cream flavors are:").lf();
    doc.ul().lf();
    for (i=0; i < num; i++) 
      doc << li << res[i] << endl;
    doc.ul().lf();
    delete [] res;
  }

  /* Approach #1: check for one of several valid responses. 
     Good if there are a short list of possible button values and
     you wish to enumerate them. */
  
  char *ages = cgi->getvar("age", 10, found );
  if ( ! found )
    doc << "Variable age not found." << br;
  else
    doc << "Age of truck : "<< ages << " (radio method)" << br;
    
  ages = cgi->getvar("age", 3, found);
  if ( ! found )
    doc << "age not found" << br;
  else
    doc << "Age of truck : " << ages << " (string method)" << br;
  
  num = 0;
  
  doc.h4("Votes (method 1)").lf();
  char **resu = cgi->getmultivar( "vote", num );
  if (! resu)
    doc << "I hate them all!" << br;
  else {	
    doc << "My preferred candidates are:" << br;
    doc.ul().lf();
    for (i=0; i < num; i++) 
      doc << li << resu[i] << endl;
    doc.ul().lf();
    delete [] resu;    
  }
  
  num = 0;

  /*   Method #2: get all the names voted for and trust them.
       This is good if the form will change more often
       than the code and invented responses are not a danger
       or can be checked in some other way. */
    
  doc.h4("Votes (method 2)").lf();

  char **resul = cgi->getmultivar("vote", num);
  if ( ! resul )
    doc << "I hate them all!" << p;
  else {
    doc << "My preferred candidates are:" << br;
    doc.ol().lf();
    for (i = 0; i < num; i ++)
      doc << li << resul[i] << endl;
    doc.ol().lf();
    delete [] resul;
  }

  // If we have writeaccess we'll write a document to disk, just to
  // show how easy it is to do.
  
  if (writeaccess){
    ofstream outdoc("out.html", ios::out);
    htmlTool filedoc( outdoc );
    filedoc.html().lf().head().title("File Document").head().lf();
    filedoc.body("bgcolor=\"#ffffff\"").lf();
    filedoc.h1("Document -&gt; File").lf();
    filedoc << "Text written, ended by &lt;br&gt;" << br;
    filedoc.body().lf().html().lf();

    doc << "Wrote document <b>out.html</b> to disk" << br;
  }

  doc << br << "In this document we ";
  writeaccess ?
    doc << "had writeaccess." << br :
    doc << "did not have writeaccess" << br;
  
  doc.body().foot().address().put( "-Cgipp by Stig Erik Sand ").lf();
  doc.link( "mailto:stig@rembrandt.uib.no", "stig@rembrandt.uib.no" ).lf();
  doc.address().foot().html();
  
  if (debugging == CGITRUE)
    cgidebug << "finished main " << endl;  

  return EXIT_SUCCESS;
}

